from abc import ABC, abstractmethod


class WxService(ABC):
    def __init__(self, credential):
        self.credential = credential

    @abstractmethod
    def get_current_weather_by_location(self, location):
        pass

    @abstractmethod
    def get_forecast_weather_by_location(self, location, period):
        pass

    @abstractmethod
    def handle_error(self, exception):
        pass



from collections import defaultdict
from itertools import repeat


from wxkit.services import WxService
from wxkit.models import ErrorModel, Wind, Temperature, Station, WeatherPoint

# from wxkit.utils.http_request import retry_request, urljoin, HTTP_METHOD
from wxkit.utils.httplib import retry_request, urljoin, HTTP_METHOD
from wxkit.utils.convertor import kph_to_mps


class PERIOD_UNIT:
    DAY = "day"
    HOUR = "hour"


def strip_data(obj):
    """
    For some data in current weather condition, eg. temperature, the data value includes
        unit in Metric and Imperial.
    In order to handle data simply, strip the data with the Metric keyword.
    """

    keyword = "Metric"
    if isinstance(obj, dict):
        try:
            obj.update(obj[keyword])
        except KeyError:
            pass

    return obj


class AccuweatherWxService(WxService):
    ENDPOINT = "http://dataservice.accuweather.com"
    FORECAST_SAMPLES = {
        PERIOD_UNIT.DAY: (1, 5, 10, 15),
        PERIOD_UNIT.HOUR: (1, 12, 24, 72, 120, 5, 10, 15),
    }

    def __init__(self, credential):
        super().__init__(credential)
        self.LOCATION_CACHE = defaultdict(dict)

    def auth_params(self, **params):
        p = dict(apikey=self.credential.apikey)
        p.update(params)
        return p

    def as_station(self, data):
        coord = data.get("GeoPosition", {})
        return Station(
            lat=coord.get("Latitude", 0),
            lon=coord.get("Longitude", 0),
            name=data.get("EnglishName", ""),
        )

    def _modelize(self, data, location):
        location_data = self.get_location(location)
        temp = data.get("Temperature", {}).get("Value", 0)
        temperature = Temperature(temp_avg=temp)
        speed = data.get("Wind", {}).get("Speed", {}).get("Value", 0)
        wind = Wind(
            speed=round(kph_to_mps(speed), 2),
            degree=data.get("Wind", {}).get("Direction", {}).get("Degrees", 0),
        )
        try:
            status = data.get("WeatherText", "") or data.get("IconPhrase", "")
            description = data.get("WeatherText", "") or data.get("IconPhrase", "")
            icon = data.get("WeatherIcon", "")
        except Exception:
            status = ""
            description = ""
            icon = ""

        pressure = data.get("Pressure", {}).get("Value", 0)
        humidity = data.get("RelativeHumidity", 0)
        rain = data.get("PrecipitationProbability", 0)
        snow = data.get("SnowProbability", 0)
        clouds = data.get("CloudCover", 0)
        time = data.get("EpochTime", 0) or data.get("EpochDateTime", 0)
        weather_point = WeatherPoint(
            station=self.as_station(location_data),
            status=status,
            description=description,
            icon=icon,
            temp=temperature,
            pressure=pressure,
            humidity=humidity,
            rain=rain,
            wind=wind,
            clouds=clouds,
            snow=snow,
            time=time,
            raw_data=data,
        )
        return weather_point

    def get_location(self, location):
        if self.LOCATION_CACHE[location]:
            return self.LOCATION_CACHE[location]

        url = urljoin(
            self.ENDPOINT,
            "locations",
            "v1",
            "cities",
            "geoposition",
            "search",
        )
        params = self.auth_params(q=f"{location.lat},{location.lon}")
        try:
            resp_data = retry_request(HTTP_METHOD.GET, url, params=params)
            self.LOCATION_CACHE[location] = resp_data
        except Exception as e:
            return self.handle_error(e)

        return resp_data

    def get_current_weather_by_location(self, location):
        location_data = self.get_location(location)
        if isinstance(location_data, ErrorModel):
            return location_data

        url = urljoin(
            self.ENDPOINT,
            "currentconditions",
            "v1",
            location_data.get("Key", ""),
        )
        params = self.auth_params(details=True)
        try:
            resp_data = retry_request(
                HTTP_METHOD.GET, url, params=params, object_hook=strip_data
            )
        except Exception as e:
            return self.handle_error(e)
        wxs = list(map(self._modelize, resp_data, repeat(location)))
        return wxs[0]

    def get_forecast_weather_by_location(
        self, location, period=1, samples=12, unit=PERIOD_UNIT.HOUR
    ):
        if unit == PERIOD_UNIT.DAY:
            forecast_type = "daily"
        elif unit == PERIOD_UNIT.HOUR:
            forecast_type = "hourly"
        else:
            return self.handle_error(
                f"Invalid period unit, must in {PERIOD_UNIT.DAY}, {PERIOD_UNIT.HOUR}"
            )

        forecast_samples = period * samples
        if forecast_samples not in self.FORECAST_SAMPLES.get(unit):
            return self.handle_error(f"Missing weather forecast for {forecast_samples}")

        location_data = self.get_location(location)
        if isinstance(location_data, ErrorModel):
            return location_data

        url = urljoin(
            self.ENDPOINT,
            "forecasts",
            "v1",
            forecast_type,
            f"{forecast_samples}{unit}",
            location_data.get("Key", ""),
        )
        params = self.auth_params(details=True, metric=True)
        try:
            resp_data = retry_request(HTTP_METHOD.GET, url, params=params)
        except Exception as e:
            return self.handle_error(e)

        wxs = list(map(self._modelize, resp_data, repeat(location)))
        return wxs

    def handle_error(self, exception):
        error = ErrorModel()
        if getattr(exception, "response", None) is not None:
            try:
                # Get error context from HTTP error if response.
                response = exception.response.json()
                error.code = response["cod"]
                error.message = response["message"]
                error.raw_data = response
            except Exception:
                error.code = exception.response.status_code
                error.message = exception.response.text
        else:
            error.message = str(exception)

        return error



from wxkit.services.openweather import OpenweatherWxService
from wxkit.services.windy import WindyWxService
from wxkit.services.accuweather import AccuweatherWxService
from wxkit import models


def test_init():
    c = models.Credential(credential={"appid": "*****"})
    s = OpenweatherWxService(c)
    assert s
    c = models.Credential(credential={"key": "*****"})
    s = WindyWxService(c)
    assert s
    c = models.Credential(credential={"apikey": "*****"})
    s = AccuweatherWxService(c)
    assert s



import logging
import os
from logging.config import dictConfig


class LOG_LEVEL:
    CRITICAL = "CRITICAL"
    ERROR = "ERROR"
    WARNING = "WARNING"
    INFO = "INFO"
    DEBUG = "DEBUG"
    NOTSET = "NOTSET"


# NOTE: Log level can be set by environments, default is debug.
_level = os.environ.get("LOG_LEVEL", LOG_LEVEL.DEBUG)

LOG_FMT = (
    "[%(asctime)s.%(msecs)03d][%(levelname)s][pid%(process)d-tid%(thread)d]"
    + "[%(module)s][%(funcName)s:%(lineno)s]: %(message)s"
)


LOGGING_CONFIG = {
    "version": 1,
    "disable_existing_loggers": True,
    "formatters": {"standard": {"format": LOG_FMT, "datefmt": "%Y-%m-%dT%H:%M:%S"}},
    "handlers": {
        "default": {
            "level": _level,
            "formatter": "standard",
            "class": "logging.StreamHandler",
            "stream": "ext://sys.stdout",  # Default is stderr
        },
    },
    "loggers": {
        # Default root logger if omitted name
        "": {
            "handlers": ["default"],
            "level": _level,
            "propagate": False,
        },
    },
}

# NOTE: Run once at startup
dictConfig(LOGGING_CONFIG)


def get_logger():
    return logging.getLogger()



#!/usr/bin/env python3

import sys
import re
from setuptools import setup, find_packages

DESC_PATTERN = re.compile(r"(# ){1}[\S]+([\s\w\.\?\!-_]+)(## ){1}.*")

if sys.version_info.major < 3:
    sys.exit("Sorry, Python2.x is not supported")

with open("README.md", encoding="UTF-8") as readme_file:
    readme = readme_file.read()
    _m = DESC_PATTERN.match(readme)
    assert _m, "Description is not matched"
    description = description = _m.group(2).strip()

with open("LICENSE", encoding="UTF-8") as license_file:
    license = license_file.read()


requirements = list()
with open("requirements.txt", "r") as fp:
    install_reqs = fp.readlines()
    requirements = [ir.rstrip() for ir in install_reqs if not ir.startswith("-")]

test_requirements = ["pytest", "tox"]

setup(
    name="wxkit",
    version="1.0.12",
    description=description,
    long_description=readme,
    url="https://github.com/returnchung/wxkit",
    author="JuiTeng Chung",
    author_email="return1225@gmail.com",
    license=license,
    python_requires=">=3",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "Environment :: MacOS X",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License"
        "Natural Language :: English",
        "Operating System :: MacOS",
        "Operating System :: MacOS :: MacOS X"
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Topic :: Utilities",
    ],
    packages=find_packages(include=["wxkit*"]),
    install_requires=requirements,
    test_suite="tests",
    tests_require=test_requirements,
    keywords="weather, wx, openweather",
    zip_safe=False,
)
